/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XLED software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XLED source distribution.
 *
 * $Id: rledstate.c,v 1.2 2002/12/11 19:06:38 brbarret Exp $
 * 
 *	Function:	- get process LED status information
 *	Accepts:	- destination node
 *			- process index
 *			- ptr state structure
 *	Returns:	- 0 or ERROR
 */

#include <events.h>
#include <ksignal.h>
#include <ledreq.h>
#include <net.h>
#include <portable.h>
#include <terror.h>
#include <typical.h>

int
lam_rledstate(node, proc, pledstate)

int4			node;
int4			proc;
struct lestate		*pledstate;

{
	struct nmsg	nhead;
	struct lereq	*req;
	struct lereply	*reply;
	int4		mask;
/*
 * Set up the request and send it.
 */
	req = (struct lereq *) nhead.nh_data;
	req->leq_src_node = (node == LOCAL) ? node : getnodeid();
	req->leq_src_event = -getpid();
	req->leq_req = LEQSTATE;
	req->leq_proc = proc;

	nhead.nh_node = node;
	nhead.nh_event = EVLEDD;
	nhead.nh_type = 0;
	nhead.nh_flags = 0;
	nhead.nh_length = 0;
	nhead.nh_msg = 0;

	if (nsend(&nhead)) {
		return(ERROR);
	}
/*
 * Receive the reply.
 */
	mask = ksigblock(sigmask(SIGUDIE) | sigmask(SIGARREST));
	nhead.nh_event = -getpid();

	if (nrecv(&nhead)) {
		ksigsetmask(mask);
		return(ERROR);
	}

	ksigsetmask(mask);
	reply = (struct lereply *) nhead.nh_data;

	if (reply->ler_ret) {
		errno = reply->ler_errno;
		return(ERROR);
	}
	else {
		pledstate->les_nleds = reply->ler_nleds;
		pledstate->les_mask = reply->ler_mask;
		return(0);
	}
}
