/*
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XLED software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XLED source distribution.
 *
 * $Id: xled.c,v 1.2 2002/12/11 19:06:38 brbarret Exp $
 * 
 *	Function:	- LAM LED server
 *			- Motif
 */

#define _NO_PROTO

#include <Xm/MessageB.h>
#include <Xm/Form.h>
#include <X11/cursorfont.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <events.h>
#include <kio.h>
#include <ledreq.h>
#include <net.h>
#include <portable.h>
#include <priority.h>
#include <terror.h>
#include <typical.h>
#include <xled.h>

/*
 * global functions
 */
void			xled_bail();
void			xled_setsize();
void			xled_setminmax();
Widget			xled_wmshell();

/*
 * external functions
 */
extern Widget		xled_ctl_build();
extern Widget		xled_view_build();
extern void		xled_view_clear();
extern void		xled_view_init();
extern void		xled_view_onoff();
extern void		xled_view_state();
extern void		_cipc_cleanup();
extern void		_lam_atexit();

/*
 * local functions
 */
static int		reqfront();
static void		reqback();

/*
 * global variables
 */
AppData			app_res;	/* application resources */

/*
 * external variables
 */
extern struct kio_t	_kio;

/*
 * local variables
 */
static struct nmsg	reqhead;	/* network message desc. */
static char		reqbuf[MAXNMSGLEN];
					/* network message body */
static Widget		toplevel;	/* toplevel window */

static int		hinitmin = -1;	/* initial drawable min height */
static int		winitmin = -1;	/* initial drawable min width */
static int		hinitmax = -1;	/* initial drawable max height */
static int		winitmax = -1;	/* initial drawable max width */
static int		hsetup = -1;	/* height setup overhead */
static int		wsetup = -1;	/* width setup overhead */

static XtResource	resources[] = {
	{ XtNrankFont, XtCRankFont,
		XmRFontStruct, sizeof(XFontStruct *),
		XtOffset(AppDataPtr, ap_rankfont),
		XmRString, XLED_RANKFONT },
	{ XtNled0Colour, XtCLed0Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[0]),
		XmRString, XLED_LED0COLOUR },
	{ XtNled1Colour, XtCLed1Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[1]),
		XmRString, XLED_LED1COLOUR },
	{ XtNled2Colour, XtCLed2Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[2]),
		XmRString, XLED_LED2COLOUR },
	{ XtNled3Colour, XtCLed3Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[3]),
		XmRString, XLED_LED3COLOUR },
	{ XtNled4Colour, XtCLed4Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[4]),
		XmRString, XLED_LED4COLOUR },
	{ XtNled5Colour, XtCLed5Colour,
		XmRString, sizeof(char *),
		XtOffset(AppDataPtr, ap_ledcolour[5]),
		XmRString, XLED_LED5COLOUR },
};

/*
 * fallback resources
 */
static String fallbacks[] = {
	"*Title: XLED",
	"*IconName: XLED",
	"*multiClickTime: 500",
	"*fontList: -*-helvetica-bold-r-normal--*-120-*-*-*-*-*-*",
	"*background: gray",
	"*ctl_bar.background: slateblue",
	"*ctl_bar.foreground: white",
	"*view_draw.background: black",
	"*view_draw.foreground: gray",
	"*rankFont: -*-helvetica-bold-r-normal--*-140-*-*-*-*-*-*",
	"*led0Colour: green",
	"*led1Colour: red",
	"*led2Colour: red",
	"*led3Colour: red",
	"*led4Colour: red",
	"*led5Colour: red",
	NULL
};

/*
 *	main
 */
int
main(argc, argv)

int			argc;
char			*argv[];

{
	Widget		topmgmt_w;
	Widget		w1, w2;
	XtAppContext	app;
	int		lamfd;

	toplevel = XtVaAppInitialize(&app, "XLED", NULL, 0,
			&argc, argv, fallbacks, NULL);

	XtGetApplicationResources(toplevel, &app_res,
			resources, XtNumber(resources), NULL, 0);
/*
 * Become a Trollius process.
 */
	if (kinit(PRCMD)) {
		terror("xled (kinit)");
		exit(errno);
	}

	_kio.ki_rtf |= RTF_SYSGEN;

	if (lpattach("xled")) lamfail("xled (lpattach)");

	_lam_atexit(xled_bail);

	topmgmt_w = XtVaCreateWidget("top_mgr",
			xmFormWidgetClass, toplevel,
			XmNmarginHeight, 2,
			XmNmarginWidth, 2,
			NULL);
/*
 * Create main application view.
 */
	w1 = xled_ctl_build(topmgmt_w);
	XtVaSetValues(w1,
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	w2 = xled_view_build(topmgmt_w);
	XtVaSetValues(w2,
		XmNtopAttachment, XmATTACH_WIDGET,
		XmNtopWidget, w1,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		NULL);

	XtManageChild(topmgmt_w);
	XSetIOErrorHandler((XIOErrorHandler) xled_bail);
/*
 * Prime a request and add file desc. to event loop.
 */
	lamfd = reqfront();
	if (lamfd < 0) lamfail("xled (bfrecvfront)");

	XtAppAddInput(app, lamfd,
			(XtPointer) XtInputReadMask, reqback, NULL);

	XtRealizeWidget(toplevel);

	xled_setminmax(winitmin, hinitmin, winitmax, hinitmax);

	XtAppMainLoop(app);
	return(0);
}

/*
 *	xled_bail
 *
 *	Function:	- exits gracefully with 0 status
 */
void
xled_bail()

{
/*
 *	lam_kexit(0);
 */
}

/*
 *	xled_wmshell
 *
 *	Function:	- returns the first WMShell ancestor
 *	Accepts:	- widget
 *	Returns:	- window manager shell
 */
Widget
xled_wmshell(w)

Widget			w;

{
	while ((w) && ( ! XtIsWMShell(w))) {
		w = XtParent(w);
	}

	return(w);
}

/*
 *	xled_setminmax
 *
 *	Function:	- set min/max toplevel shell size
 *	Accepts:	- min width
 *			- min height
 *			- max width
 *			- max height
 */
void
xled_setminmax(wmin, hmin, wmax, hmax)

int			wmin;
int			hmin;
int			wmax;
int			hmax;

{
	Dimension	height;
	Dimension	width;
/*
 * The first time around, just store the drawable sizes.
 */
	if (winitmin < 0) {
		winitmin = wmin;
		hinitmin = hmin;
		winitmax = wmax;
		hinitmax = hmax;
	}
	else {
/*
 * The second time around, compute the width/height window overhead.
 */
		if (wsetup < 0) {
			XtVaGetValues(toplevel, XmNwidth, &width,
					XmNheight, &height, NULL);

			wsetup = width - wmax;
			hsetup = height - hmax;
		}
/*
 * Set the size boundaries.
 */
		XtVaSetValues(toplevel,
			XmNminWidth, wsetup + wmin,
			XmNminHeight, hsetup + hmin,
			XmNmaxWidth, wsetup + wmax,
			XmNmaxHeight, hsetup + hmax,
			NULL);
	}
}

/*
 *	xled_setsize
 *
 *	Function:	- set toplevel shell size
 *	Accepts:	- width
 *			- height
 */
void
xled_setsize(width, height)

int			width;
int			height;

{
	XResizeWindow(XtDisplay(toplevel), XtWindow(toplevel),
		(unsigned) wsetup + width, (unsigned) hsetup + height);
}

/*
 *	xled_seticon
 *
 *	Function:	- set the tool icon
 *	Accepts:	- icon pixmap
 */
void
xled_seticon(iconpix)

Pixmap			iconpix;

{
	Window		win;
	Window		root;
	Display		*disp;
	int		x, y;
	unsigned int	width, height, bwidth, depth;

	disp = XtDisplay(toplevel);
	XtVaGetValues(toplevel, XmNiconWindow, &win, NULL);

	if (! win) {
		if (! XGetGeometry(disp, iconpix, &root, &x, &y,
				&width, &height, &bwidth, &depth) ||
			!(win = XCreateSimpleWindow(disp, root, 0, 0,
				width, height, (unsigned int) 0,
				CopyFromParent, CopyFromParent))) {

			XtVaSetValues(toplevel, XmNiconPixmap, iconpix, NULL);
			return;
		}

		XtVaSetValues(toplevel, XmNiconWindow, win, NULL);
	}

	XSetWindowBackgroundPixmap(disp, win, iconpix);
	XClearWindow(disp, win);
}

/*
 *	reqfront
 *
 *	Function:	- setup a receive request
 *	Returns:	- socket or ERROR
 */
static int
reqfront()

{
	reqhead.nh_event = EVLEDD;
	reqhead.nh_type = 0;
	reqhead.nh_flags = 0;
	reqhead.nh_length = sizeof(reqbuf);
	reqhead.nh_msg = reqbuf;

	return(bfrecvfront(&reqhead));
}

/*
 *	reqback
 *
 *	Function:	- finish the receive request
 *			- handle the request
 *			- setup a new receive request
 */
static void
reqback()

{
	struct lereq	*req;
	struct lereply	*reply;
	int		ret;

	ret = bfrecvback(&reqhead);
/*
 * If interrupted, retry.
 */
	if ( ((ret > 0) && ((ret & _kio.ki_sigretry) == ret)) ||
				((ret < 0) && (errno == ETIMEDOUT))) {
		reqfront();
		return;
	}

	if (ret) {
		return;
	}
/*
 * Handle received request.
 */
	req = (struct lereq *) reqhead.nh_data;

	switch(req->leq_req) {

	case LEQINIT:
		xled_view_init(req->leq_proc, req->leq_nprocs);
		break;

	case LEQON:
		xled_view_onoff(req->leq_proc,
				req->leq_nprocs, req->leq_mask, 1);
		break;

	case LEQOFF:
		xled_view_onoff(req->leq_proc,
				req->leq_nprocs, req->leq_mask, 0);
		break;

	case LEQSTATE:
		reply = (struct lereply *) reqhead.nh_data;

		reqhead.nh_node = req->leq_src_node;
		reqhead.nh_event = req->leq_src_event;
		reqhead.nh_type = 0;
		reqhead.nh_flags = 0;
		reqhead.nh_length = 0;
		reqhead.nh_msg = 0;

		xled_view_state(req->leq_proc,
				&reply->ler_nleds, &reply->ler_mask);

		reply->ler_ret = 0;

		if (nsend(&reqhead)) {
			return;
		}
		break;

	default:
		break;
	}

	reqfront();
}
